//-------------------------------------------------------------------------------------
//	ping_pong_muc
//-------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------
//	Phil le   16/01/2013  
//-------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------
//	includes
//-------------------------------------------------------------------------------------
#define classe_e
#include"base.h"
#undef classe_e
//-------------------------------------------------------------------------------------
//gestion des interruptions
//-------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------
//reception d'un message en provenance de la liaison RS232
//-------------------------------------------------------------------------------------
ISR(USART_RXC_vect)
{


	
	cli();
	
	//v=rs232_rx_8();

	if(rs232_rx_valeur()==1)
	{
		envoi=1;
	}
	else
	{
		envoi=0;
	}


	sei();
}
//-------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------
//programme principal
//-------------------------------------------------------------------------------------
int main(void)
{	


	DDRC=255;
	sens=1;
	cpt=1;
	envoi=0;
	
	//init rs232 en bidirectionnel à 115200 bauds
	rs232_init(16,1);
	
	sei();

	init();
	
	//boucle infinie
	while(1)
	{
		if (envoi==0)
		{
			chenillard();
		}
		else
		{
			rs232_tx_valeur(valeur_flottante,3);
			envoi=0;			
		}
	}
		
	return 0;
}

//-------------------------------------------------------------------------------------
//routines
//-------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------
//init
int init(void)
{
	char mess[50];
	
	strcpy(mess,">>>>>>>>>>\n");
	rs232_tx_chaine(strlen(mess),mess);		

	strcpy(mess,"ping_pong_chaine\n");
	rs232_tx_chaine(strlen(mess),mess);		

	strcpy(mess,"<<<<<<<<<<\n\n");
	rs232_tx_chaine(strlen(mess),mess);		


}
//-------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------
//chenillard
void chenillard(void)
{
	
	if (sens==1)
	{
		if (cpt<128)
		{
			cpt=cpt*2;
		}
		else
		{
			sens=0;
		}
	}
	else
	{
		if (cpt>1)
		{
			cpt=cpt/2;
		}
		else
		{
			sens=1;
		}
	}

	PORTC=cpt;
	
	_delay_ms(10.0);
	_delay_ms(10.0);
}
//-------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------
//initialisation de la transmission
//-------------------------------------------------------------------------------------
void rs232_init(unsigned char baud,unsigned char isr_rx)
{
	cli();
	
	//reglage de la vitesse
 	UBRRH=(unsigned char)(baud>>8);
	UBRRL=(unsigned char)baud;
	
	//autorisation transmission reception
	UCSRB=(1<<RXEN)|(1<<TXEN);
	
	//reglage protocole (8 bits de donnees, 2 bits de stop)
	UCSRC=(1<<URSEL)|(1<<USBS)|(3<<UCSZ0);
	
	//reglage mode asynchrone
 	UCSRC=UCSRC | 64;
 	
 	//reglage double debit
	UCSRA=UCSRA | 2;
	
	//autorisation interruption sur reception
	if (isr_rx!=0)
	{
		UCSRB = UCSRB | 128;
	}
	
	sei();
}
//-------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------
//reception d'un mot de 8 bits
//-------------------------------------------------------------------------------------
unsigned char rs232_rx_8(void)
{
	//attente de reception
	while (!(UCSRA & (1<<RXC)))
	{
	}
	
	//reception et retour
	return UDR;
}
//-------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------
//emission d'un mot de 8 bits
//-------------------------------------------------------------------------------------
void rs232_tx_8(unsigned char v)
{
	//cli();
	
	//attente buffer emission vide
	while (!(UCSRA & (1<<UDRE)))
	{
	}

	//envoi
	UDR=v;
	
	PORTC=v;
	
	//sei();
}
//-------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------
//reception d'un mot de 16 bits
//-------------------------------------------------------------------------------------
uint16_t rs232_rx_16(void)
{
	volatile uint16_t res;
	volatile unsigned char msb;
	volatile unsigned char lsb;
	
	msb=rs232_rx_8();
	lsb=rs232_rx_8();
	
	res=msb*256+lsb;
	
	return res;
}
//-------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------
//emission d'un mot de 16 bits
//-------------------------------------------------------------------------------------
void rs232_tx_16(uint16_t v)
{
	volatile unsigned char msb;
	volatile unsigned char lsb;
	
	msb=v/256;
	lsb=v-msb*256;

	rs232_tx_8(msb);	
	rs232_tx_8(lsb);	
}
//-------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------
//emission d'un mot de 32 bits
//-------------------------------------------------------------------------------------
//reception 32
float rs232_rx_32(void)
{
	volatile float *res;
	volatile uint8_t val[4];
	volatile uint8_t i;
	
	for (i=0;i<4;i++)
	{
		val[i]=rs232_rx_8();
	}

	res=(float*)val;
		
	return *res;
}
//-------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------
//emission d'un mot de 32 bits
//-------------------------------------------------------------------------------------
//emission 32
void rs232_tx_32(float v)
{
	volatile uint8_t *val;
	volatile uint8_t i;
	
	val=(uint8_t*)&v;
	
	for (i=0;i<4;i++)
	{
		rs232_tx_8(*val);
		val++;
	}
}
//-------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------
//emission d'une chaine
//-------------------------------------------------------------------------------------
void rs232_tx_chaine(uint8_t l,char *c)
{
	uint8_t i;
	
	for (i=0;i<l;i++)
	{
		rs232_tx_8(c[i]);

	//	PORTC=c[i];
	//	delai(1000);
		
	}
}
//-------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------
//emission d'une valeur quelconque
//-------------------------------------------------------------------------------------
void rs232_tx_valeur(float v,int nd)
{
	int E;
	int D;
	int S;
	float V;
	float mul;
	int i;
	
	//determination du signe
	if (v<0)
	{
		S=-1;
		V=-v;
	}
	else
	{
		S=1;
		V=v;
	}
	
	//partie entiere
	E=floor(V);
	
	//coef multiplicateur pour extraction des decimales
	mul=1;
	
	for (i=0;i<nd;i++)
	{
		mul=mul*10;
	}
	
	//partie decimale
	D=floor(V*mul-E*mul);
	
	trace_ent(E,S);
	trace_dec(D,nd);
	
	rs232_tx_8(10);
	
}
//-------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------
//trace_ent
//-------------------------------------------------------------------------------------
void trace_ent(int v,int s)
{
	int i;
	int mag;
	int V;
	int div;
	int A;
	

	V=v;
	
	//magnitude de v
	if (v!=0)
	{
		mag=floor(log10(v));
	}
	else
	{
		mag=0;
	}
	
	//coef diviseur
	div=1;
	
	for (i=0;i<mag;i++)
	{
		div=div*10;
	}
	
	
	//signe
	if (s==-1)
	{
//		printf("%c",'-');
		rs232_tx_8('-');

	}
	
	for (i=mag;i>=0;i--)
	{
		A=floor(V/div);
		
//		printf("%d",A);
		rs232_tx_8(A+48);
		
		V=V-A*div;
		div=div/10;
	}
}

//-------------------------------------------------------------------------------------
//trace_dec
//-------------------------------------------------------------------------------------
void trace_dec(int v,int nd)
{
	int V;
	int i;
	int div;
	int A;
	
	V=v;
	
	if (V!=0)
	{
//		printf("%c",'.');
		rs232_tx_8('.');	
		
		//coef diviseur
		div=1;
	
		for (i=0;i<nd;i++)
		{
			div=div*10;
		}
	
		for (i=nd;i>0;i--)
		{
			div=(int)(div/10);

			A=floor((float)V/(float)div);
		
//			printf("%d",A);
			rs232_tx_8(A+48);
		
			V=V-A*div;
		}
	}
}



//delai
void delai(uint16_t d)
{
	uint16_t i;
	
	for (i=0;i<d;i++)
	{
		_delay_ms(1.0);
	}
}


//conversion dec_char
void dec_char(uint8_t v)
{
	
}


//reception d'une valeur quelconque
uint8_t rs232_rx_valeur(void)
{
	uint8_t i;
	uint8_t n;
	uint8_t nc;
	uint8_t suite;
	unsigned char c[30];
	char mess[50];
	uint8_t valid;
	uint8_t cpt;
	int8_t signe;
	int8_t posv;
	float v;
	float mul;

	uint8_t ct;
	
	n=30;
	suite=1;
	i=0;
	
	//reception
	while (suite==1 && i<n)
	{
		ct=rs232_rx_8();
		
		if (ct==10)
		{
			suite=0;
		}

		c[i]=(unsigned char)ct;
		i++;
	}
	
	nc=i-1;
	

	//analyse
	
	//detection du signe
	if (c[0]=='-')
	{
		signe=-1;
		c[0]='0';
	}
	else
	{
		signe=1;
	}

	valid=1;
	cpt=0;
	
	//test si caractere possible 0123456789.
	//et comptage du nombre de virgules
	posv=-1;
	
	for (i=0;i<nc;i++)
	{
		if (c[i]<'0' || c[i]>'9')
		{
			if (c[i]!='.')
			{
				valid=0;
			}
		}
		
		if (c[i]=='.')
		{
			cpt++;
			posv=i;
		}
	}
	
	if (cpt>1)
	{
		valid=0;
	}
	
	if (valid==1)
	{
		//formation de la valeur
		mul=1;
	
		//cas general de la virgule placée n'importe ou
		for (i=0;i<posv-1;i++)
		{
			mul=mul*10;
		}
	
		//cas particulier de la virgule placée en premier
		if (posv==0)
		{
			mul=0.1;
		}
	
		//cas particulier sans virgule
		if (posv==-1)
		{
			mul=1;
	
			for (i=0;i<nc-1;i++)
			{
				mul=mul*10;
			}
		}
	
		//conversion
		v=0;
	
		for (i=0;i<nc;i++)
		{
			if (c[i]!='.')
			{
				v=v+(c[i]-48)*mul;
				mul=mul/10;
			}
		}
		
		//depassement de capacité
		if (v>=32768)
		{
			valeur_flottante=0;
			valid=0;	
			
			strcpy(mess,"valeur trop grande\n");
			rs232_tx_chaine(strlen(mess),mess);		
		}
		else
		{
			v=v*signe;
			valeur_flottante=v;
		}
	}
	else
	{
		valeur_flottante=0;
		strcpy(mess,"pas compris !\n");
		rs232_tx_chaine(strlen(mess),mess);		
	}
	
	return valid;
}


